<?php
/**
 * @file
 * Custom module functionality.
 */
/**
 * API function to enable a notice with proper validation and messages.  Use
 * this function as a callback for user facing calls.
 *
 * Wrapper for site_notice_update_status().
 */
function site_notice_enable_notice($notice, $force = FALSE) {
  $args = array('%name' => $notice->title);
  $pos  = variable_get('site_notice_default_position', SiteNoticeEntity::TOP);

  // Access check
  if ($force || site_notice_access('edit', $notice)) {
    if ($notice->status) {
      drupal_set_message(t('Site notice %name is already enabled.', array('%name' => $notice->title)), 'warning');
    }
    else {
      if (site_notice_update_status($notice, $pos)) {
        drupal_set_message(t('Notice %name has been enabled.', $args));
      }
      else {
        drupal_set_message(t('Error enabling notice %name.', $args), 'error');
      }
    }
  }
  else {
    drupal_set_message(t('You do not have privelages to enable the site notice %name.', array('%name' => $notice->title)), 'error');
  }

  drupal_goto(drupal_get_destination());
}

/**
 * API function to enable a notice with proper validation and messages.  Use
 * this function as a callback for user facing calls.
 *
 * Wrapper for site_notice_update_status().
 */
function site_notice_disable_notice($notice, $force = FALSE) {
  $args = array('%name' => $notice->title);

  // Access check
  if ($force || site_notice_access('edit', $notice)) {
    if (!$notice->status) {
      drupal_set_message(t('Site notice %name is already disabled.', $args), 'warning');
    }
    else {
      if (site_notice_update_status($notice, 0)) {
        drupal_set_message(t('Notice %name has been disabled.', $args));
      }
      else {
        drupal_set_message(t('Error disabling notice %name.', $args), 'error');
      }
    }
  }
  else {
    drupal_set_message(t('You do not have privelages to disable the site notice %name.', $args), 'error');
  }

  drupal_goto(drupal_get_destination());
}

/**
 * API call to directly change the status of a notice.  Only use this if you
 * have already checked for permission.
 *
 * @see site_notice_enable()
 * @see site_notice_disable()
 */
function site_notice_update_status($notice, $status) {
  $notice->status = (int) $status;
  return $notice->save();
}

/**
 * Filter notices.
 *
 * @todo: Load all active notices in one call.
 */
function site_notice_filter_notices() {
  $return   = array();
  $notices  = site_notice_load_multiple(FALSE, array('status' => SiteNoticeEntity::TOP));
  $notices += site_notice_load_multiple(FALSE, array('status' => SiteNoticeEntity::BOTTOM));

  foreach ($notices as $notice) {
    if (_site_notice_filter_path($notice) && _site_notice_filter_role($notice)) {
      $return[$notice->status][$notice->snid] = $notice;
    }
  }

  return $return;
}

/**
 * Helper to determine if a notice passes the path filter.
 */
function _site_notice_filter_path($notice) {
  $s = $notice->settings['path-filter'];

  $pages_match = drupal_match_path($_GET['q'], $s['value']);

  // Check whether the page filter is inclusionary or exclusionary
  if ($s['status'] == SiteNoticeEntity::FILTER_INCLUDE && !$pages_match) {
    return FALSE;
  }
  elseif ($s['status'] == SiteNoticeEntity::FILTER_EXCLUDE && $pages_match) {
    return FALSE;
  }

  return TRUE;
}

/**
 * Helper to determine if a notice passes the role filter.
 */
function _site_notice_filter_role($notice) {
  global $user;
  $s = $notice->settings['role-filter'];

  // Get an array of roles checked in the notice and assigned to the active user.
  $matching_roles = array();
  foreach ($s['value'] as $rid => $active) {
    if ($active && isset($user->roles[$rid])) {
      $matching_roles[$rid] = $user->roles[$rid];
    }
  }

  // Check whether the role filter is inclusionary or exclusionary
  if ($s['status'] == SiteNoticeEntity::FILTER_INCLUDE && empty($matching_roles)) {
    return FALSE;
  }
  elseif ($s['status'] == SiteNoticeEntity::FILTER_EXCLUDE && !empty($matching_roles)) {
    return FALSE;
  }

  return TRUE;
}

/**
 * Helper to return the text value for status.
 */
function site_notice_status_text($status = NULL) {
  $values = array(
    SiteNoticeEntity::DISABLED => t('Disabled'),
    SiteNoticeEntity::TOP      => t('Top'),
    SiteNoticeEntity::BOTTOM   => t('Bottom'),
  );

  return (is_null($status) ? $values : (isset($values[$status]) ? $values[$status] : ''));
}
